#include <util/twi.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include "i2c.h"
#include "ipmb.h"

#define TWCR_ACK TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);  
#define TWCR_NACK TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(0<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|(0<<TWWC);
#define TWCR_RESET TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA)|(0<<TWSTA)|(1<<TWSTO)|(0<<TWWC);  

static volatile unsigned char databuf[12];
static volatile uint8_t pos = 0x00;

void i2c_init()
{
	TWAR = BMC_ADDR & 0xfe;
	TWDR = 0x00;
	TWCR &= ~((1<<TWSTA) | (1<<TWSTO));
	TWCR |= ((1<<TWEA) | (1<<TWEN) | (1<<TWIE)); 
	printf("Status: 0x%02x\n", TW_STATUS);
	PORTC = 0x03;
}

void i2c_send(unsigned char *buf, int len)
{
	uint8_t old_TWCR = TWCR;
	uint8_t old_SREG = SREG;
	int i;

	cli();

	TWCR = ((1<<TWINT) | (1<<TWSTA) | (1<<TWEN)); /* Send start */

	while(!(TWCR & (1<<TWINT))) {}
	if ((TW_STATUS & 0xf8) != TW_START)
		goto out;

	TWDR = buf[0]; /* SLA_W */
	TWCR = ((1<<TWINT) | (1<<TWEN));

	while(!(TWCR & (1<<TWINT))) {}
	if ((TW_STATUS & 0xf8) != TW_MT_SLA_ACK)
		goto out;
	
	for(i = 1; i < len; i++) {
		TWDR = buf[i]; /* Send Data */
		TWCR = ((1<<TWINT) | (1<<TWEN));

		while(!(TWCR & (1<<TWINT))) {}
		if ((TW_STATUS & 0xf8) != TW_MT_DATA_ACK)
			goto out;
	}

	TWCR = ((1<<TWINT) | (1<<TWEN) | (1<<TWSTO));

	printf("I2C Data sent 0x%02x\n", TW_STATUS);


out:	
	TWDR = 0x00;
	TWCR = old_TWCR;
	SREG = old_SREG;
}

ISR (TWI_vect, ISR_BLOCK)
{
	switch (TW_STATUS) {
		case TW_SR_SLA_ACK:
#ifdef DEBUG
			printf("I2C: Slave 0x%02x adressed\n", TWDR);
#endif
			pos = 0x00;
			databuf[pos] = TWDR;
			pos++;
			TWCR_ACK;
			break;

		case TW_SR_DATA_ACK:
#ifdef DEBUG
			printf("I2C: Data received: 0x%02x\n", TWDR);
#endif
			databuf[pos] = TWDR;
			pos++;
			TWCR_ACK;
			break;

		case TW_SR_STOP:
#ifdef DEBUG
			printf("I2C: STOP received\n");
#endif
			decode_ipmb_pkt((unsigned char*)databuf, pos);
			pos = 0x00;
			TWCR_RESET;
			break;

		default:
			printf("I2C: Unimplemented status 0x%02x\n", TW_STATUS);
			TWCR_RESET;
			break;
	}
}
